/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.exec.service.test;

import javax.xml.namespace.QName;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.types.URI.MalformedURIException;
import org.globus.exec.utils.audit.AuditUtil;
import org.globus.exec.utils.ManagedJobConstants;
import org.globus.transfer.reliable.service.RFTConstants;

/**
 * Test both methods of AuditUtil with different EPRs: 1. EPR from GRAM
 * (ResourceID as resource key name) 2. EPR from RFT (TransferKey as resource
 * key name) 3. A fictitious EPR 4. An EPR with value null
 * 
 * The correctness of the transforming algorithm in AuditUtil which transforms
 * the ResourceKey isn't tested here.
 */

public class AuditUtilTest extends TestCase
{
    public AuditUtilTest(String name)
    {
        super(name);
    }

    public static void main(String[] args)
    {
        TestSuite testSuite = null;

        // run all tests defined here
        if (testSuite == null) {
            testSuite = new TestSuite(AuditUtilTest.class);
        }

        junit.textui.TestRunner.run(testSuite);
    }

    /**
     * create the EPR's used for the test. Use EPRs with different resource key
     * names in the ReferenceProperties part because it used to cause problems
     * when the EPR didn't have the GRAM resource key name. EPRs with resource
     * key qnames from GRAM, RFT and a fictional service are tested. Also an EPR
     * with value null is tested
     */
    public void setUp()
    {

        // create the EPRs
        gramEpr = new EndpointReferenceType();
        rftEpr = new EndpointReferenceType();
        myEpr = new EndpointReferenceType();

        gramServiceURI = 
            "https://127.0.0.1:8443/wsrf/services/ManagedJobFactoryService";
        rftServiceURI = 
            "https://127.0.0.1:8443/wsrf/services/ReliableFileTransferService";
        myServiceURI = 
            "https://127.0.0.1:8443/wsrf/services/MyService";
        myNamespace = 
            "http://schemas.xmlsoap.org/ws/2004/03/addressing";
        dummyResourceKeyValue = 
            "123456";

        // Constructor of class Address throws MalformedURIException in case
        // of an error
        try {
            // prepare EPR with GRAM-specific resource key name
            QName gramResourceKeyQName = new QName(
                    ManagedJobConstants.RESOURCE_KEY_QNAME.getNamespaceURI(),
                    ManagedJobConstants.RESOURCE_KEY_QNAME.getLocalPart());
            MessageElement gramMessageElement = new MessageElement(
                    gramResourceKeyQName, dummyResourceKeyValue);
            ReferencePropertiesType gramRpt = new ReferencePropertiesType();
            gramRpt.set_any(new MessageElement[] { gramMessageElement });
            gramEpr.setAddress(new Address(gramServiceURI));
            gramEpr.setProperties(gramRpt);

            // prepare EPR with RFT-specific resource key name
            QName rftResourceKeyQName = new QName(RFTConstants.RFT_KEY_QNAME
                    .getNamespaceURI(), RFTConstants.RFT_KEY_QNAME
                    .getLocalPart());
            MessageElement rftMessageElement = new MessageElement(
                    rftResourceKeyQName, dummyResourceKeyValue);
            ReferencePropertiesType rftRpt = new ReferencePropertiesType();
            rftRpt.set_any(new MessageElement[] { rftMessageElement });
            rftEpr.setAddress(new Address(rftServiceURI));
            rftEpr.setProperties(rftRpt);

            // prepare EPR with arbitrary resource key name
            String resourceKeyQName = "MyResourceKeyName";
            QName myQName = new QName(myNamespace, resourceKeyQName);
            MessageElement myMessageElement = new MessageElement(myQName,
                    dummyResourceKeyValue);
            ReferencePropertiesType myRpt = new ReferencePropertiesType();
            myRpt.set_any(new MessageElement[] { myMessageElement });
            myEpr.setAddress(new Address(myServiceURI));
            myEpr.setProperties(myRpt);

        } catch (MalformedURIException e) {
            System.err.println("AuditUtilTest.setup: " + e.toString());
        }
    }

    /**
     * test AuditUtil.eprToGridId(EndpointReferenceType epr) throws Exception
     */
    public void testEprToGridId_1()
    {
        String gridId = null;

        // call with GRAM-EPR
        try {
            gridId = AuditUtil.eprToGridId(gramEpr);
        } catch (Exception e) {
            fail("Exception thrown by AuditUtil.eprToGridId():" + e.toString());
        }
        assertTrue(gridId != null);
        assertTrue(gridId.startsWith(gramServiceURI + "?"));

        // call with RFT-EPR
        gridId = null;
        try {
            gridId = AuditUtil.eprToGridId(rftEpr);
        } catch (Exception e) {
            fail("Exception thrown by AuditUtil.eprToGridId():" + e.toString());
        }
        assertTrue(gridId != null);
        assertTrue(gridId.startsWith(rftServiceURI + "?"));

        // call with unknown EPR: exception expected
        gridId = null;
        try {
            gridId = AuditUtil.eprToGridId(myEpr);
            fail();
        } catch (Exception e) {
        }

        // call with null EPR: exception expected
        gridId = null;
        try {
            gridId = AuditUtil.eprToGridId(null);
            fail();
        } catch (Exception e) {
        }

    }

    /**
     * test AuditUtil.eprToGridId(EndpointReferenceType epr, MessageElement me)
     * throws Exception
     */
    public void testEprToGridId_2()
    {
        String gridId = null;

        // call with GRAM-EPR
        try {
            gridId = AuditUtil.eprToGridId(gramEpr, gramEpr.getProperties()
                    .get_any()[0]);
        } catch (Exception e) {
            fail("Exception thrown by AuditUtil.eprToGridId():" + e.toString());
        }
        assertTrue(gridId != null);
        assertTrue(gridId.startsWith(gramServiceURI + "?"));

        // call with RFT-EPR
        gridId = null;
        try {
            gridId = AuditUtil.eprToGridId(rftEpr, rftEpr.getProperties()
                    .get_any()[0]);
        } catch (Exception e) {
            fail("Exception thrown by AuditUtil.eprToGridId():" + e.toString());
        }
        assertTrue(gridId != null);
        assertTrue(gridId.startsWith(rftServiceURI + "?"));

        // call with unknown EPR: should work here
        gridId = null;
        try {
            gridId = AuditUtil.eprToGridId(myEpr, myEpr.getProperties()
                    .get_any()[0]);
        } catch (Exception e) {
            fail("Exception thrown by AuditUtil.eprToGridId():" + e.toString());
        }
        assertTrue(gridId != null);
        assertTrue(gridId.startsWith(myServiceURI + "?"));

        // call with mixed values: EPR from RFT and
        // MessageElement from GRAM: should work
        gridId = null;
        try {
            gridId = AuditUtil.eprToGridId(rftEpr, gramEpr.getProperties()
                    .get_any()[0]);
        } catch (Exception e) {
            fail("Exception thrown by AuditUtil.eprToGridId():" + e.toString());
        }
        assertTrue(gridId != null);
        assertTrue(gridId.startsWith(rftServiceURI + "?"));

        // call with null EPR and an arbitrary MessageElement:
        // gridId must be null
        gridId = null;
        try {
            gridId = AuditUtil.eprToGridId(null, myEpr.getProperties()
                    .get_any()[0]);
        } catch (Exception e) {
            fail("Exception thrown by AuditUtil.eprToGridId():" + e.toString());
        }
        assertTrue(gridId == null);

    }

    private EndpointReferenceType gramEpr;
    private EndpointReferenceType rftEpr;
    private EndpointReferenceType myEpr;
    private String gramServiceURI;
    private String rftServiceURI;
    private String myServiceURI;
    private String myNamespace;
    private String dummyResourceKeyValue;
}
