/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.exec.service.test;

import junit.framework.TestSuite;
import java.io.File;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileInputStream;
import org.apache.axis.message.MessageElement;
import org.globus.exec.generated.JobDescriptionType;
import org.globus.exec.generated.ExtensionsType;
import org.globus.wsrf.encoding.ObjectSerializer;
import javax.xml.namespace.QName; 

/**
 * Check if the extension element is processed when specified by the user in
 * the job description document.
 * This is done by using the 'logfile' extension element.
 * A temporary file is created which is added to a simple job description as
 * value of the logfile extensions element. It's expected that the Perl job
 * manager writes logging information to that file.
 * After job submission a check is done whether the file includes logging
 * statements from the Perl job manager.
 * The file is removed at the end of the test if the JVM exits normally.
 * NOTE: An invalid logfile value (like /root/cantWriteHere.txt)
 * does not cause the job to fail, the logfile simply will not be written in
 * that case
 */
public class JDDExtensionsTest extends JobSubmissionTest
{
    public JDDExtensionsTest(String name)
    {
        super(name);
    }

    public static void main(String[] args)
    {
        TestSuite testSuite = null;

        // run all tests defined here
        if (testSuite == null) {
            testSuite = new TestSuite(JDDExtensionsTest.class);
        }

        junit.textui.TestRunner.run(testSuite);
    }


    /**
     *  Check that the logfile extension works correctly
     */
    public void testLogfileExtension() throws Exception
    {
        File tempFile = null;
        
        // create a temporary file for the value of the extensions
        // element 'logfile'
        try {
            tempFile = File.createTempFile("gram",".tmp");
            tempFile.deleteOnExit();
        } catch (Exception e) {
            fail("can't create temporary file: " + e.getMessage());
        }
        
        // create a simple job description and add extensions element 'logfile'
        JobDescriptionType jobDescription = getDefaultJobDescription();
        ExtensionsType extensions = new ExtensionsType();
        // This does not work: then namespace is not set and the extension
        // element is not recognized.  Why?
        MessageElement messageElement = new MessageElement(
                new QName("","logfile"),
                tempFile.getAbsolutePath());
        messageElement.setType(org.apache.axis.Constants.XSD_STRING);

        extensions.set_any(new MessageElement[] {messageElement});
        jobDescription.setExtensions(extensions);
        
        // submit the job and wait for termination 
        submitJobAndWaitForCompletion(jobDescription);
        
        // read temporary file and search for strings written by the 
        // Perl job manager. These Strings are "EXTENSIONS_HANDLER" and
        // "JM_SCRIPT". If these strings can't be found in the logfile
        // this test failed.
        BufferedReader br =
            new BufferedReader(
                new InputStreamReader(
                    new FileInputStream(tempFile)));
        
        String line = "";
        String extensionHandlerSearchToken = "EXTENSIONS_HANDLER";
        String jobManagerSearchToken = "JM_SCRIPT";
        boolean extensionHandlerOk = false;
        boolean jobManagerOk = false;
        
        while((line = br.readLine()) != null) {
            if (line.indexOf(extensionHandlerSearchToken) > -1) {
                extensionHandlerOk = true;
            }
            if (line.indexOf(jobManagerSearchToken) > -1) {
                jobManagerOk = true;
            }
        }
        
        // In case of failure the job description will be displayed
        // on the error channel
        if (!extensionHandlerOk || !jobManagerOk) {
            fail("Job Manager logfile does not contain the " +
                 "information it should\nsent job description:\n" +
                 ObjectSerializer.toString(
                         jobDescription,
                         new QName("http://www.globus.org/namespaces/2004/10/gram/job/description",
                                   "job")));
        } 
    }

}
