/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.exec.service.test;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;
import java.util.Enumeration;
import junit.framework.TestSuite;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.globus.exec.generated.JobDescriptionType;
import org.globus.exec.generated.StateEnumeration;

/**
 * Check if the mapping of substitution variables defined in the JNDI
 * configuration of a scheduler works correctly. Tests are: 1. submit a simple
 * job with a valid substitution variable in the argument element of the job
 * description 2. submit a simple job with an invalid substitution variable in
 * the argument element of the job description
 * 
 * The substitution variable file which is used in this test is the one from the
 * Fork scheduler ($GLOBUS_LOCATION/etc/gram-service-Fork/jndi-config.xml)
 * because it always exists. The user who runs this test must have read access
 * to that file and to the substitution variables file which is specified in
 * JDNI configuration.
 * 
 */
public class JDDSubstitutionVariablesTest extends JobSubmissionTest
{
    public JDDSubstitutionVariablesTest(String name)
    {
        super(name);
        substitutionVariables = new Properties();
    }

    public static void main(String[] args)
    {
        TestSuite testSuite = null;

        // run all tests defined here
        if (testSuite == null) {
            testSuite = new TestSuite(JDDSubstitutionVariablesTest.class);
        }

        junit.textui.TestRunner.run(testSuite);
    }

    /**
     * Get the environment variable $GLOBUS_LOCATION, set the path to the JNDI
     * configuration file of the Fork scheduler and read the substitution
     * variable file to a Properties object
     */
    public void setUp() throws Exception
    {
        globusLocation = System.getProperty("GLOBUS_LOCATION");
        if (globusLocation == null) {
            fail("$GLOBUS_LOCATION not defined in environment");
        }
        jndiConfigurationFile = globusLocation + File.separator + "etc"
                + File.separator + "gram-service-Fork" + File.separator
                + "jndi-config.xml";

        readSubstitutionVariablesFromFile();
        super.setUp();
    }

    /**
     * Submit a simple job with a valid substitution variable in the argument
     * element of the job description. Should work.
     */
    public void testExistingVariable() throws Exception
    {
        Enumeration e = substitutionVariables.propertyNames();
        if (e.hasMoreElements()) {
            // Submit job with the first variable in the argument element
            // of the job description document
            String variableName = (String) e.nextElement();
            variableName = "${" + variableName + "}";
            JobDescriptionType jobDescription = getDefaultJobDescription();
            jobDescription.setArgument(new String[] { variableName });
            submitJobAndWaitForCompletion(jobDescription);
        } else {
            // No variables specified in the properties. There should
            // be at least $GLOBUS_LOCATION
            fail("No substitution variables at all defined for the scheduler");
        }
    }

    /**
     * Submit a simple job with an invalid substitution variable in the argument
     * element of the job description. Should fail. To get an invalid
     * substitution variable a String of "X"s is generated until it is no longer
     * found in the substitution properties
     * 
     */
    public void testNonExistingVariable() throws Exception
    {
        String invalidVariable = "X";
        while (substitutionVariables.getProperty(invalidVariable) != null) {
            invalidVariable = invalidVariable.concat("X");
        }
        invalidVariable = "${" + invalidVariable + "}";
        JobDescriptionType jobDescription = getDefaultJobDescription();
        jobDescription.setArgument(new String[] { invalidVariable });

        // submit job and wait for the first state notification
        // which should be 'Failed' because of the invalid variable
        this.submitJobWithStateSubscription(jobDescription);
        int expectedNotificationCount = 1;
        waitForCount(expectedNotificationCount, getWaitTimeMillis());
        assertTrue("Job with invalid substitution variable succeeded",
                wasNotified(StateEnumeration.Failed));
    }

    /**
     * Max time in millis to wait for a certain event.
     */
    protected int getWaitTimeMillis()
    {
        return 30000;
    }

    /**
     * Read the value of 'substitutionDefinitionsFile' from the file
     * $GLOBUS_LOCATION/etc/gram-service-Fork/jndi-config.xml and store the
     * variable mappings defined in the properties file in a Properties object
     */
    private void readSubstitutionVariablesFromFile()
    {
        try {
            DocumentBuilderFactory docBuilderFactory = 
                DocumentBuilderFactory.newInstance();
            DocumentBuilder docBuilder =
                docBuilderFactory.newDocumentBuilder();
            Document doc = docBuilder.parse(new File(jndiConfigurationFile));

            // normalize text representation
            doc.getDocumentElement().normalize();

            // get all parameters
            NodeList listOfParameters = doc.getElementsByTagName("parameter");

            // loop over all parameters
            for (int s = 0; s < listOfParameters.getLength(); s++) {
                Node parameterNode = listOfParameters.item(s);

                if (parameterNode.getNodeType() == Node.ELEMENT_NODE) {
                    Element parameterElement = (Element) parameterNode;

                    // get name and value of the parameter
                    NodeList nameList = parameterElement
                            .getElementsByTagName("name");
                    String name = getNodeValueAsString((Element) nameList
                            .item(0));

                    // if the name is substitutionDefinitionsFile read the file
                    // into a Properties element
                    if (name.equals("substitutionDefinitionsFile")) {
                        NodeList valueList = parameterElement
                                .getElementsByTagName("value");
                        String propertiesFileName = getNodeValueAsString(
                                (Element) valueList.item(0));
                        substitutionVariables.load(new FileInputStream(
                                propertiesFileName));
                    }
                }
            }
        } catch (SAXParseException err) {
            System.out.println("** Parsing error" + ", line "
                    + err.getLineNumber() + ", uri " + err.getSystemId());
            System.out.println(" " + err.getMessage());
        } catch (SAXException e) {
            Exception x = e.getException();
            ((x == null) ? e : x).printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Helper method to read the values from a 'name' and 'value' node in the
     * jndiConfigurationFile. Comments inside are ignored
     */
    public String getNodeValueAsString(Element element)
    {
        StringBuffer sb = new StringBuffer();
        NodeList nodeList = element.getChildNodes();

        // only TEXT_NODE elements expected, cut the rest (like COMMENT_NODE)
        for (int i = 0; i < nodeList.getLength(); i++) {
            Node node = nodeList.item(i);
            if (node.getNodeType() == Node.TEXT_NODE) {
                sb.append(node.getNodeValue());
            }
        }
        return sb.toString().trim();
    }

    private String globusLocation;
    private String jndiConfigurationFile;
    private Properties substitutionVariables;
}
